@@ -4,7 +4,7 @@ See header file: geos-x.y.z/capi/geos_c.h
 '''
 
 from ctypes import CFUNCTYPE, POINTER, c_void_p, c_char_p, \
-    c_size_t, c_byte, c_char, c_uint, c_int, c_double
+    c_size_t, c_byte, c_char, c_uint, c_int, c_double, py_object
 
 # Derived pointer types
 c_size_t_p = POINTER(c_size_t)
@@ -66,6 +66,9 @@ def prototype(lgeos, geos_version):
     lgeos.GEOSCoordSeq_clone.restype = c_void_p
     lgeos.GEOSCoordSeq_clone.argtypes = [c_void_p]
 
+    lgeos.GEOSGeom_clone.restype = c_void_p
+    lgeos.GEOSGeom_clone.argtypes = [c_void_p]
+
     lgeos.GEOSCoordSeq_destroy.restype = None
     lgeos.GEOSCoordSeq_destroy.argtypes = [c_void_p]
 
@@ -206,6 +209,11 @@ def prototype(lgeos, geos_version):
         lgeos.GEOSPolygonize_full.restype = c_void_p
         lgeos.GEOSPolygonize_full.argtypes = [c_void_p, c_void_p, c_void_p, c_void_p]
 
+    if geos_version >= (3, 4, 0):
+        lgeos.GEOSDelaunayTriangulation.restype = c_void_p
+        lgeos.GEOSDelaunayTriangulation.argtypes = [c_void_p, c_double, c_int]
+
+
     lgeos.GEOSLineMerge.restype = c_void_p
     lgeos.GEOSLineMerge.argtypes = [c_void_p]
 
@@ -268,6 +276,10 @@ def prototype(lgeos, geos_version):
     lgeos.GEOSisRing.restype = c_byte
     lgeos.GEOSisRing.argtypes = [c_void_p]
 
+    if geos_version >= (3, 3, 0):
+        lgeos.GEOSisClosed.restype = c_byte
+        lgeos.GEOSisClosed.argtypes = [c_void_p]
+
     lgeos.GEOSHasZ.restype = c_byte
     lgeos.GEOSHasZ.argtypes = [c_void_p]
 
@@ -453,3 +465,26 @@ def prototype(lgeos, geos_version):
 
         lgeos.GEOSFree.restype = None
         lgeos.GEOSFree.argtypes = [c_void_p]
+
+    if geos_version >= (3, 4, 0):
+        lgeos.GEOSNearestPoints.restype = c_void_p
+        lgeos.GEOSNearestPoints.argtypes = [c_void_p, c_void_p]
+
+    if geos_version >= (3, 4, 2):
+        lgeos.GEOSQueryCallback = CFUNCTYPE(None, c_void_p, c_void_p)
+
+        lgeos.GEOSSTRtree_query.argtypes = [c_void_p, c_void_p, lgeos.GEOSQueryCallback, py_object]
+        lgeos.GEOSSTRtree_query.restype = None
+
+        lgeos.GEOSSTRtree_create.argtypes = [c_int]
+        lgeos.GEOSSTRtree_create.restype = c_void_p
+
+        lgeos.GEOSSTRtree_insert.argtypes = [c_void_p, c_void_p, py_object]
+        lgeos.GEOSSTRtree_insert.restype = None
+
+        lgeos.GEOSSTRtree_remove.argtypes = [c_void_p, c_void_p, py_object]
+        lgeos.GEOSSTRtree_remove.restype = None
+
+        lgeos.GEOSSTRtree_destroy.argtypes = [c_void_p]
+        lgeos.GEOSSTRtree_destroy.restype = None
+
