@@ -36,19 +36,22 @@ else:
 def load_dll(libname, fallbacks=None):
     lib = find_library(libname)
     if lib is not None:
-        return CDLL(lib)
-    else:
-        if fallbacks is not None:
-            for name in fallbacks:
-                try:
-                    return CDLL(name)
-                except OSError:
-                    # move on to the next fallback
-                    pass
-        # the end
-        raise OSError(
-            "Could not find library %s or load any of its variants %s" % (
-                libname, fallbacks or []))
+        try:
+            return CDLL(lib)
+        except OSError:
+            pass
+    if fallbacks is not None:
+        for name in fallbacks:
+            try:
+                return CDLL(name)
+            except OSError:
+                # move on to the next fallback
+                pass
+    # No shared library was loaded. Raise OSError.
+    raise OSError(
+        "Could not find library %s or load any of its variants %s" % (
+            libname, fallbacks or []))
+
 
 if sys.platform.startswith('linux'):
     _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
@@ -380,6 +383,10 @@ class WKBWriter(object):
     _lgeos = None
     _writer = None
 
+    # EndianType enum in ByteOrderValues.h
+    _ENDIAN_BIG = 0
+    _ENDIAN_LITTLE = 1
+
     # Establish default output setting
     defaults = {'output_dimension': 3}
 
@@ -396,11 +403,13 @@ class WKBWriter(object):
     @property
     def big_endian(self):
         """Byte order is big endian, True (default) or False"""
-        return bool(self._lgeos.GEOSWKBWriter_getByteOrder(self._writer))
+        return (self._lgeos.GEOSWKBWriter_getByteOrder(self._writer) ==
+            self._ENDIAN_BIG)
 
     @big_endian.setter
     def big_endian(self, value):
-        self._lgeos.GEOSWKBWriter_setByteOrder(self._writer, bool(value))
+        self._lgeos.GEOSWKBWriter_setByteOrder(
+            self._writer, self._ENDIAN_BIG if value else self._ENDIAN_LITTLE)
 
     @property
     def include_srid(self):
@@ -706,11 +715,29 @@ class LGEOS330(LGEOS320):
         attr.__name__ = func.__name__
         setattr(self, key, attr)
 
+        for pred in (self.GEOSisClosed,):
+            pred.func.errcheck = errcheck_predicate
+
         self.methods['unary_union'] = self.GEOSUnaryUnion
+        self.methods['is_closed'] = self.GEOSisClosed
         self.methods['cascaded_union'] = self.methods['unary_union']
 
 
-if geos_version >= (3, 3, 0):
+class LGEOS340(LGEOS330):
+    """Proxy for GEOS 3.4.0-CAPI-1.8.0
+    """
+    geos_version = (3, 4, 0)
+    geos_capi_version = (1, 8, 0)
+
+    def __init__(self, dll):
+        super(LGEOS340, self).__init__(dll)
+        self.methods['delaunay_triangulation'] = self.GEOSDelaunayTriangulation
+        self.methods['nearest_points'] = self.GEOSNearestPoints
+
+
+if geos_version >= (3, 4, 0):
+    L = LGEOS340
+elif geos_version >= (3, 3, 0):
     L = LGEOS330
 elif geos_version >= (3, 2, 0):
     L = LGEOS320
@@ -723,7 +750,7 @@ else:
 
 lgeos = L(_lgeos)
 
+def cleanup(proxy):
+    del proxy
 
-@atexit.register
-def cleanup():
-    lgeos.__del__()
+atexit.register(cleanup, lgeos)
